Method: OodCore::Job::Script#rerunnable

Defined in:
lib/ood_core/job/script.rb

#rerunnableBoolean? (readonly)

Note:

This SHOULD NOT be used to let the application denote the checkpointability of a job

Whether job can safely be restarted by the resource manager, for example on node failure or some other re-scheduling event

Returns:

  • (Boolean, nil)

    whether job can be restarted



29
30
31
# File 'lib/ood_core/job/script.rb', line 29

def rerunnable
  @rerunnable
end