Class: OodCore::Job::QueueInfo
- Inherits:
-
Object
- Object
- OodCore::Job::QueueInfo
- Defined in:
- lib/ood_core/job/queue_info.rb
Overview
QueueInfo is information about a given queue on a scheduler.
Instance Attribute Summary collapse
-
#allow_accounts ⇒ Object
readonly
The accounts that are allowed to use this queue.
-
#deny_accounts ⇒ Object
readonly
The accounts that are not allowed to use this queue.
-
#name ⇒ Object
(also: #to_s)
readonly
The name of the queue.
-
#qos ⇒ Object
readonly
The QoSes associated with this queue.
Instance Method Summary collapse
-
#initialize(**opts) ⇒ QueueInfo
constructor
A new instance of QueueInfo.
- #to_h ⇒ Object
Constructor Details
#initialize(**opts) ⇒ QueueInfo
20 21 22 23 24 25 |
# File 'lib/ood_core/job/queue_info.rb', line 20 def initialize(**opts) @name = opts.fetch(:name, 'unknown') @qos = opts.fetch(:qos, []) @allow_accounts = opts.fetch(:allow_accounts, nil) @deny_accounts = opts.fetch(:deny_accounts, []) end |
Instance Attribute Details
#allow_accounts ⇒ Object (readonly)
The accounts that are allowed to use this queue.
nil means ALL accounts are allowed.
15 16 17 |
# File 'lib/ood_core/job/queue_info.rb', line 15 def allow_accounts @allow_accounts end |
#deny_accounts ⇒ Object (readonly)
The accounts that are not allowed to use this queue.
18 19 20 |
# File 'lib/ood_core/job/queue_info.rb', line 18 def deny_accounts @deny_accounts end |
#name ⇒ Object (readonly) Also known as: to_s
The name of the queue.
6 7 8 |
# File 'lib/ood_core/job/queue_info.rb', line 6 def name @name end |
#qos ⇒ Object (readonly)
The QoSes associated with this queue
10 11 12 |
# File 'lib/ood_core/job/queue_info.rb', line 10 def qos @qos end |
Instance Method Details
#to_h ⇒ Object
27 28 29 30 31 32 |
# File 'lib/ood_core/job/queue_info.rb', line 27 def to_h instance_variables.map do |var| name = var.to_s.gsub('@', '').to_sym [name, send(name)] end.to_h end |